@@ -1,12 +1,13 @@
 import flask
 import werkzeug
+from ddtrace.vendor import debtcollector
 from ddtrace.vendor.wrapt import wrap_function_wrapper as _w
 
 from ddtrace import compat
 from ddtrace import config, Pin
 
 from ...constants import ANALYTICS_SAMPLE_RATE_KEY, SPAN_MEASURED_KEY
-from ...ext import SpanTypes, http
+from ...ext import SpanTypes
 from ...internal.logger import get_logger
 from ...propagation.http import HTTPPropagator
 from ...utils.wrappers import unwrap as _u
@@ -16,26 +17,27 @@ from .wrappers import wrap_function, wrap_signal
 
 log = get_logger(__name__)
 
-FLASK_ENDPOINT = 'flask.endpoint'
-FLASK_VIEW_ARGS = 'flask.view_args'
-FLASK_URL_RULE = 'flask.url_rule'
-FLASK_VERSION = 'flask.version'
+FLASK_ENDPOINT = "flask.endpoint"
+FLASK_VIEW_ARGS = "flask.view_args"
+FLASK_URL_RULE = "flask.url_rule"
+FLASK_VERSION = "flask.version"
 
 # Configure default configuration
-config._add('flask', dict(
-    # Flask service configuration
-    _default_service="flask",
-    app='flask',
-
-    collect_view_args=True,
-    distributed_tracing_enabled=True,
-    template_default_name='<memory>',
-    trace_signals=True,
-
-    # We mark 5xx responses as errors, these codes are additional status codes to mark as errors
-    # DEV: This is so that if a user wants to see `401` or `403` as an error, they can configure that
-    extra_error_codes=set(),
-))
+config._add(
+    "flask",
+    dict(
+        # Flask service configuration
+        _default_service="flask",
+        app="flask",
+        collect_view_args=True,
+        distributed_tracing_enabled=True,
+        template_default_name="<memory>",
+        trace_signals=True,
+        # We mark 5xx responses as errors, these codes are additional status codes to mark as errors
+        # DEV: This is so that if a user wants to see `401` or `403` as an error, they can configure that
+        extra_error_codes=set(),
+    ),
+)
 
 
 # Extract flask version into a tuple e.g. (0, 12, 1) or (1, 0, 2)
@@ -50,8 +52,8 @@ config._add('flask', dict(
 #      (0, 9) == (0, 9)
 #      (0, 9, 0) != (0, 9)
 #      (0, 8, 5) <= (0, 9)
-flask_version_str = getattr(flask, '__version__', '0.0.0')
-flask_version = tuple([int(i) for i in flask_version_str.split('.')])
+flask_version_str = getattr(flask, "__version__", "0.0.0")
+flask_version = tuple([int(i) for i in flask_version_str.split(".")])
 
 
 def patch():
@@ -59,177 +61,171 @@ def patch():
     Patch `flask` module for tracing
     """
     # Check to see if we have patched Flask yet or not
-    if getattr(flask, '_datadog_patch', False):
+    if getattr(flask, "_datadog_patch", False):
         return
-    setattr(flask, '_datadog_patch', True)
+    setattr(flask, "_datadog_patch", True)
 
     Pin().onto(flask.Flask)
 
     # flask.app.Flask methods that have custom tracing (add metadata, wrap functions, etc)
-    _w('flask', 'Flask.wsgi_app', traced_wsgi_app)
-    _w('flask', 'Flask.dispatch_request', request_tracer('dispatch_request'))
-    _w('flask', 'Flask.preprocess_request', request_tracer('preprocess_request'))
-    _w('flask', 'Flask.add_url_rule', traced_add_url_rule)
-    _w('flask', 'Flask.endpoint', traced_endpoint)
-    _w('flask', 'Flask._register_error_handler', traced_register_error_handler)
+    _w("flask", "Flask.wsgi_app", traced_wsgi_app)
+    _w("flask", "Flask.dispatch_request", request_tracer("dispatch_request"))
+    _w("flask", "Flask.preprocess_request", request_tracer("preprocess_request"))
+    _w("flask", "Flask.add_url_rule", traced_add_url_rule)
+    _w("flask", "Flask.endpoint", traced_endpoint)
+    _w("flask", "Flask._register_error_handler", traced_register_error_handler)
 
     # flask.blueprints.Blueprint methods that have custom tracing (add metadata, wrap functions, etc)
-    _w('flask', 'Blueprint.register', traced_blueprint_register)
-    _w('flask', 'Blueprint.add_url_rule', traced_blueprint_add_url_rule)
+    _w("flask", "Blueprint.register", traced_blueprint_register)
+    _w("flask", "Blueprint.add_url_rule", traced_blueprint_add_url_rule)
 
     # flask.app.Flask traced hook decorators
     flask_hooks = [
-        'before_request',
-        'before_first_request',
-        'after_request',
-        'teardown_request',
-        'teardown_appcontext',
+        "before_request",
+        "before_first_request",
+        "after_request",
+        "teardown_request",
+        "teardown_appcontext",
     ]
     for hook in flask_hooks:
-        _w('flask', 'Flask.{}'.format(hook), traced_flask_hook)
-    _w('flask', 'after_this_request', traced_flask_hook)
+        _w("flask", "Flask.{}".format(hook), traced_flask_hook)
+    _w("flask", "after_this_request", traced_flask_hook)
 
     # flask.app.Flask traced methods
     flask_app_traces = [
-        'process_response',
-        'handle_exception',
-        'handle_http_exception',
-        'handle_user_exception',
-        'try_trigger_before_first_request_functions',
-        'do_teardown_request',
-        'do_teardown_appcontext',
-        'send_static_file',
+        "process_response",
+        "handle_exception",
+        "handle_http_exception",
+        "handle_user_exception",
+        "try_trigger_before_first_request_functions",
+        "do_teardown_request",
+        "do_teardown_appcontext",
+        "send_static_file",
     ]
     for name in flask_app_traces:
-        _w('flask', 'Flask.{}'.format(name), simple_tracer('flask.{}'.format(name)))
+        _w("flask", "Flask.{}".format(name), simple_tracer("flask.{}".format(name)))
 
     # flask static file helpers
-    _w('flask', 'send_file', simple_tracer('flask.send_file'))
+    _w("flask", "send_file", simple_tracer("flask.send_file"))
 
     # flask.json.jsonify
-    _w('flask', 'jsonify', traced_jsonify)
+    _w("flask", "jsonify", traced_jsonify)
 
     # flask.templating traced functions
-    _w('flask.templating', '_render', traced_render)
-    _w('flask', 'render_template', traced_render_template)
-    _w('flask', 'render_template_string', traced_render_template_string)
+    _w("flask.templating", "_render", traced_render)
+    _w("flask", "render_template", traced_render_template)
+    _w("flask", "render_template_string", traced_render_template_string)
 
     # flask.blueprints.Blueprint traced hook decorators
     bp_hooks = [
-        'after_app_request',
-        'after_request',
-        'before_app_first_request',
-        'before_app_request',
-        'before_request',
-        'teardown_request',
-        'teardown_app_request',
+        "after_app_request",
+        "after_request",
+        "before_app_first_request",
+        "before_app_request",
+        "before_request",
+        "teardown_request",
+        "teardown_app_request",
     ]
     for hook in bp_hooks:
-        _w('flask', 'Blueprint.{}'.format(hook), traced_flask_hook)
+        _w("flask", "Blueprint.{}".format(hook), traced_flask_hook)
 
     # flask.signals signals
-    if config.flask['trace_signals']:
+    if config.flask["trace_signals"]:
         signals = [
-            'template_rendered',
-            'request_started',
-            'request_finished',
-            'request_tearing_down',
-            'got_request_exception',
-            'appcontext_tearing_down',
+            "template_rendered",
+            "request_started",
+            "request_finished",
+            "request_tearing_down",
+            "got_request_exception",
+            "appcontext_tearing_down",
         ]
         # These were added in 0.11.0
         if flask_version >= (0, 11):
-            signals.append('before_render_template')
+            signals.append("before_render_template")
 
         # These were added in 0.10.0
         if flask_version >= (0, 10):
-            signals.append('appcontext_pushed')
-            signals.append('appcontext_popped')
-            signals.append('message_flashed')
+            signals.append("appcontext_pushed")
+            signals.append("appcontext_popped")
+            signals.append("message_flashed")
 
         for signal in signals:
-            module = 'flask'
+            module = "flask"
 
             # v0.9 missed importing `appcontext_tearing_down` in `flask/__init__.py`
             #  https://github.com/pallets/flask/blob/0.9/flask/__init__.py#L35-L37
             #  https://github.com/pallets/flask/blob/0.9/flask/signals.py#L52
             # DEV: Version 0.9 doesn't have a patch version
-            if flask_version <= (0, 9) and signal == 'appcontext_tearing_down':
-                module = 'flask.signals'
+            if flask_version <= (0, 9) and signal == "appcontext_tearing_down":
+                module = "flask.signals"
 
             # DEV: Patch `receivers_for` instead of `connect` to ensure we don't mess with `disconnect`
-            _w(module, '{}.receivers_for'.format(signal), traced_signal_receivers_for(signal))
+            _w(module, "{}.receivers_for".format(signal), traced_signal_receivers_for(signal))
 
 
 def unpatch():
-    if not getattr(flask, '_datadog_patch', False):
+    if not getattr(flask, "_datadog_patch", False):
         return
-    setattr(flask, '_datadog_patch', False)
+    setattr(flask, "_datadog_patch", False)
 
     props = [
         # Flask
-        'Flask.wsgi_app',
-        'Flask.dispatch_request',
-        'Flask.add_url_rule',
-        'Flask.endpoint',
-        'Flask._register_error_handler',
-
-        'Flask.preprocess_request',
-        'Flask.process_response',
-        'Flask.handle_exception',
-        'Flask.handle_http_exception',
-        'Flask.handle_user_exception',
-        'Flask.try_trigger_before_first_request_functions',
-        'Flask.do_teardown_request',
-        'Flask.do_teardown_appcontext',
-        'Flask.send_static_file',
-
+        "Flask.wsgi_app",
+        "Flask.dispatch_request",
+        "Flask.add_url_rule",
+        "Flask.endpoint",
+        "Flask._register_error_handler",
+        "Flask.preprocess_request",
+        "Flask.process_response",
+        "Flask.handle_exception",
+        "Flask.handle_http_exception",
+        "Flask.handle_user_exception",
+        "Flask.try_trigger_before_first_request_functions",
+        "Flask.do_teardown_request",
+        "Flask.do_teardown_appcontext",
+        "Flask.send_static_file",
         # Flask Hooks
-        'Flask.before_request',
-        'Flask.before_first_request',
-        'Flask.after_request',
-        'Flask.teardown_request',
-        'Flask.teardown_appcontext',
-
+        "Flask.before_request",
+        "Flask.before_first_request",
+        "Flask.after_request",
+        "Flask.teardown_request",
+        "Flask.teardown_appcontext",
         # Blueprint
-        'Blueprint.register',
-        'Blueprint.add_url_rule',
-
+        "Blueprint.register",
+        "Blueprint.add_url_rule",
         # Blueprint Hooks
-        'Blueprint.after_app_request',
-        'Blueprint.after_request',
-        'Blueprint.before_app_first_request',
-        'Blueprint.before_app_request',
-        'Blueprint.before_request',
-        'Blueprint.teardown_request',
-        'Blueprint.teardown_app_request',
-
+        "Blueprint.after_app_request",
+        "Blueprint.after_request",
+        "Blueprint.before_app_first_request",
+        "Blueprint.before_app_request",
+        "Blueprint.before_request",
+        "Blueprint.teardown_request",
+        "Blueprint.teardown_app_request",
         # Signals
-        'template_rendered.receivers_for',
-        'request_started.receivers_for',
-        'request_finished.receivers_for',
-        'request_tearing_down.receivers_for',
-        'got_request_exception.receivers_for',
-        'appcontext_tearing_down.receivers_for',
-
+        "template_rendered.receivers_for",
+        "request_started.receivers_for",
+        "request_finished.receivers_for",
+        "request_tearing_down.receivers_for",
+        "got_request_exception.receivers_for",
+        "appcontext_tearing_down.receivers_for",
         # Top level props
-        'after_this_request',
-        'send_file',
-        'jsonify',
-        'render_template',
-        'render_template_string',
-        'templating._render',
+        "after_this_request",
+        "send_file",
+        "jsonify",
+        "render_template",
+        "render_template_string",
+        "templating._render",
     ]
 
     # These were added in 0.11.0
     if flask_version >= (0, 11):
-        props.append('before_render_template.receivers_for')
+        props.append("before_render_template.receivers_for")
 
     # These were added in 0.10.0
     if flask_version >= (0, 10):
-        props.append('appcontext_pushed.receivers_for')
-        props.append('appcontext_popped.receivers_for')
-        props.append('message_flashed.receivers_for')
+        props.append("appcontext_pushed.receivers_for")
+        props.append("appcontext_popped.receivers_for")
+        props.append("message_flashed.receivers_for")
 
     for prop in props:
         # Handle 'flask.request_started.receivers_for'
@@ -239,11 +235,11 @@ def unpatch():
         #  https://github.com/pallets/flask/blob/0.9/flask/__init__.py#L35-L37
         #  https://github.com/pallets/flask/blob/0.9/flask/signals.py#L52
         # DEV: Version 0.9 doesn't have a patch version
-        if flask_version <= (0, 9) and prop == 'appcontext_tearing_down.receivers_for':
+        if flask_version <= (0, 9) and prop == "appcontext_tearing_down.receivers_for":
             obj = flask.signals
 
-        if '.' in prop:
-            attr, _, prop = prop.partition('.')
+        if "." in prop:
+            attr, _, prop = prop.partition(".")
             obj = getattr(obj, attr, object())
         _u(obj, prop)
 
@@ -264,7 +260,7 @@ def traced_wsgi_app(pin, wrapped, instance, args, kwargs):
     request = werkzeug.Request(environ)
 
     # Configure distributed tracing
-    if config.flask.get('distributed_tracing_enabled', False):
+    if config.flask.get("distributed_tracing_enabled", False):
         propagator = HTTPPropagator()
         context = propagator.extract(request.headers)
         # Only need to activate the new context if something was propagated
@@ -275,7 +271,7 @@ def traced_wsgi_app(pin, wrapped, instance, args, kwargs):
     #   GET /
     #   POST /save
     # We will override this below in `traced_dispatch_request` when we have a `RequestContext` and possibly a url rule
-    resource = u'{} {}'.format(request.method, request.path)
+    resource = u"{} {}".format(request.method, request.path)
     with pin.tracer.trace(
         "flask.request",
         service=trace_utils.int_service(pin, config.flask),
@@ -297,7 +293,7 @@ def traced_wsgi_app(pin, wrapped, instance, args, kwargs):
         #   On the bright side, this works in all versions of Flask (or any WSGI app actually)
         def _wrap_start_response(func):
             def traced_start_response(status_code, headers):
-                code, _, _ = status_code.partition(' ')
+                code, _, _ = status_code.partition(" ")
                 try:
                     code = int(code)
                 except ValueError:
@@ -310,23 +306,38 @@ def traced_wsgi_app(pin, wrapped, instance, args, kwargs):
                 #      we still want `GET /product/<int:product_id>` grouped together,
                 #      even if it is a 404
                 if not s.get_tag(FLASK_ENDPOINT) and not s.get_tag(FLASK_URL_RULE):
-                    s.resource = u'{} {}'.format(request.method, code)
+                    s.resource = u"{} {}".format(request.method, code)
+
+                trace_utils.set_http_meta(s, config.flask, status_code=code, response_headers=headers)
+
+                extra_error_codes = config.flask.get("extra_error_codes")
+                if extra_error_codes:
+                    debtcollector.deprecate(
+                        (
+                            "ddtrace.config.flask['extra_error_codes'] is now deprecated, "
+                            "use ddtrace.config.http_server.error_statuses"
+                        ),
+                        removal_version="0.47.0",
+                    )
+                    if code in extra_error_codes:
+                        s.error = 1
 
-                s.set_tag(http.STATUS_CODE, code)
-                if 500 <= code < 600:
-                    s.error = 1
-                elif code in config.flask.get('extra_error_codes', set()):
-                    s.error = 1
                 return func(status_code, headers)
+
             return traced_start_response
+
         start_response = _wrap_start_response(start_response)
 
         # DEV: We set response status code in `_wrap_start_response`
         # DEV: Use `request.base_url` and not `request.url` to keep from leaking any query string parameters
-        s.set_tag(http.URL, request.base_url)
-        s.set_tag(http.METHOD, request.method)
-        if config.flask.trace_query_string:
-            s.set_tag(http.QUERY_STRING, compat.to_unicode(request.query_string))
+        trace_utils.set_http_meta(
+            s,
+            config.flask,
+            method=request.method,
+            url=request.base_url,
+            query=compat.to_unicode(request.query_string),
+            request_headers=request.headers,
+        )
 
         return wrapped(environ, start_response)
 
@@ -338,7 +349,7 @@ def traced_blueprint_register(wrapped, instance, args, kwargs):
     This wrapper just ensures the blueprint has a pin, either set manually on
     itself from the user or inherited from the application
     """
-    app = kwargs.get('app', args[0])
+    app = kwargs.get("app", args[0])
     # Check if this Blueprint has a pin, otherwise clone the one from the app onto it
     pin = Pin.get_from(instance)
     if not pin:
@@ -363,6 +374,7 @@ def traced_blueprint_add_url_rule(wrapped, instance, args, kwargs):
 
 def traced_add_url_rule(wrapped, instance, args, kwargs):
     """Wrapper for flask.app.Flask.add_url_rule to wrap all views attached to this app"""
+
     def _wrap(rule, endpoint=None, view_func=None, **kwargs):
         if view_func:
             # TODO: `if hasattr(view_func, 'view_class')` then this was generated from a `flask.views.View`
@@ -376,17 +388,18 @@ def traced_add_url_rule(wrapped, instance, args, kwargs):
 
 def traced_endpoint(wrapped, instance, args, kwargs):
     """Wrapper for flask.app.Flask.endpoint to ensure all endpoints are wrapped"""
-    endpoint = kwargs.get('endpoint', args[0])
+    endpoint = kwargs.get("endpoint", args[0])
 
     def _wrapper(func):
         # DEV: `wrap_function` will call `func_name(func)` for us
         return wrapped(endpoint)(wrap_function(instance, func, resource=endpoint))
+
     return _wrapper
 
 
 def traced_flask_hook(wrapped, instance, args, kwargs):
     """Wrapper for hook functions (before_request, after_request, etc) are properly traced"""
-    func = kwargs.get('f', args[0])
+    func = kwargs.get("f", args[0])
     return wrapped(wrap_function(instance, func))
 
 
@@ -396,7 +409,7 @@ def traced_render_template(wrapped, instance, args, kwargs):
     if not pin or not pin.enabled():
         return wrapped(*args, **kwargs)
 
-    with pin.tracer.trace('flask.render_template', span_type=SpanTypes.TEMPLATE):
+    with pin.tracer.trace("flask.render_template", span_type=SpanTypes.TEMPLATE):
         return wrapped(*args, **kwargs)
 
 
@@ -406,7 +419,7 @@ def traced_render_template_string(wrapped, instance, args, kwargs):
     if not pin or not pin.enabled():
         return wrapped(*args, **kwargs)
 
-    with pin.tracer.trace('flask.render_template_string', span_type=SpanTypes.TEMPLATE):
+    with pin.tracer.trace("flask.render_template_string", span_type=SpanTypes.TEMPLATE):
         return wrapped(*args, **kwargs)
 
 
@@ -425,17 +438,20 @@ def traced_render(wrapped, instance, args, kwargs):
         return wrapped(*args, **kwargs)
 
     def _wrap(template, context, app):
-        name = getattr(template, 'name', None) or config.flask.get('template_default_name')
+        name = getattr(template, "name", None) or config.flask.get("template_default_name")
         span.resource = name
-        span.set_tag('flask.template_name', name)
+        span.set_tag("flask.template_name", name)
         return wrapped(*args, **kwargs)
+
     return _wrap(*args, **kwargs)
 
 
 def traced_register_error_handler(wrapped, instance, args, kwargs):
     """Wrapper to trace all functions registered with flask.app.register_error_handler"""
+
     def _wrap(key, code_or_exception, f):
         return wrapped(key, code_or_exception, wrap_function(instance, f))
+
     return _wrap(*args, **kwargs)
 
 
@@ -457,34 +473,37 @@ def request_tracer(name):
 
             # DEV: This name will include the blueprint name as well (e.g. `bp.index`)
             if not span.get_tag(FLASK_ENDPOINT) and request.endpoint:
-                span.resource = u'{} {}'.format(request.method, request.endpoint)
+                span.resource = u"{} {}".format(request.method, request.endpoint)
                 span.set_tag(FLASK_ENDPOINT, request.endpoint)
 
             if not span.get_tag(FLASK_URL_RULE) and request.url_rule and request.url_rule.rule:
-                span.resource = u'{} {}'.format(request.method, request.url_rule.rule)
+                span.resource = u"{} {}".format(request.method, request.url_rule.rule)
                 span.set_tag(FLASK_URL_RULE, request.url_rule.rule)
 
-            if not span.get_tag(FLASK_VIEW_ARGS) and request.view_args and config.flask.get('collect_view_args'):
+            if not span.get_tag(FLASK_VIEW_ARGS) and request.view_args and config.flask.get("collect_view_args"):
                 for k, v in request.view_args.items():
-                    span.set_tag(u'{}.{}'.format(FLASK_VIEW_ARGS, k), v)
+                    span.set_tag(u"{}.{}".format(FLASK_VIEW_ARGS, k), v)
         except Exception:
             log.debug('failed to set tags for "flask.request" span', exc_info=True)
 
-        with pin.tracer.trace('flask.{}'.format(name), service=trace_utils.int_service(pin, config.flask, pin)):
+        with pin.tracer.trace("flask.{}".format(name), service=trace_utils.int_service(pin, config.flask, pin)):
             return wrapped(*args, **kwargs)
+
     return _traced_request
 
 
 def traced_signal_receivers_for(signal):
     """Wrapper for flask.signals.{signal}.receivers_for to ensure all signal receivers are traced"""
+
     def outer(wrapped, instance, args, kwargs):
-        sender = kwargs.get('sender', args[0])
+        sender = kwargs.get("sender", args[0])
         # See if they gave us the flask.app.Flask as the sender
         app = None
         if isinstance(sender, flask.Flask):
             app = sender
         for receiver in wrapped(*args, **kwargs):
             yield wrap_signal(app, signal, receiver)
+
     return outer
 
 
@@ -493,5 +512,5 @@ def traced_jsonify(wrapped, instance, args, kwargs):
     if not pin or not pin.enabled():
         return wrapped(*args, **kwargs)
 
-    with pin.tracer.trace('flask.jsonify'):
+    with pin.tracer.trace("flask.jsonify"):
         return wrapped(*args, **kwargs)
