@@ -2,6 +2,8 @@ import requests
 
 from ddtrace.vendor.wrapt import wrap_function_wrapper as _w
 
+from ddtrace import config, Pin
+
 from .connection import _wrap_send
 
 
@@ -10,8 +12,10 @@ class TracedSession(requests.Session):
     You can use it if you want a finer grained control for your
     HTTP clients.
     """
+
     pass
 
 
 # always patch our `TracedSession` when imported
-_w(TracedSession, 'send', _wrap_send)
+_w(TracedSession, "send", _wrap_send)
+Pin(_config=config.requests).onto(TracedSession)
